#include "hart_flash.h"

#include <drivers/flash/adi_flash.h>

//=========================== define ==========================================

#define FLASH_PAGE_LEN8     (2048) // in bytes
#define WRITE_LEN           ((sizeof(hart_flash_memImage_t) / 8 + 1) * 8) // must be a a multiply of 8

//=========================== variables =======================================

ADI_FEE_HANDLE     hFlash;
uint8_t            flash_DevMem[ADI_FEE_MEMORY_SIZE];

//=========================== public ==========================================

void hart_flash_write(const void* address, void* data, uint8_t dataLen, bool incCfgChangeCnt)
{
		static bool firstRun = true;
	  uint32_t err;
		uint32_t pageNumber;
		uint32_t dataOffset;
		uint8_t  pageImage[FLASH_PAGE_LEN8]; // TODO this is not very effective, we do not need all the memory
		hart_flash_memImage_t *memImage = (hart_flash_memImage_t*) pageImage;
	
		if (firstRun)
		{
				adi_fee_Open(0, flash_DevMem, ADI_FEE_MEMORY_SIZE, &hFlash);
				firstRun = false;
		}
	
		// get position which we wil write data into
		if (address != NULL)
				dataOffset = (uint8_t*)address - (uint8_t*) HART_FLASH_ADDR;
		else
				dataOffset = 0;
	
		// copy flash to local image
		memcpy(pageImage, (void*) HART_FLASH_ADDR , sizeof(hart_flash_memImage_t));
	
		// get flash page number, that will be read, erased and written to
		adi_fee_GetPageNumber(hFlash, HART_FLASH_ADDR, &pageNumber);
	
		// delete only this (one) page
		adi_fee_PageErase(hFlash, pageNumber, pageNumber, &err);
	
		// save new data to local image
		if (data != NULL)
				memcpy(&pageImage[dataOffset], data, dataLen);
		else
				memset(&pageImage[dataOffset], 0, dataLen);
		
		// increment configuration changed counter and flag the change
		if (incCfgChangeCnt == true)
		{
				memImage->cfgCounter.value++;
				memImage->cfgCounter.flags[0] = 0xFF;
				memImage->cfgCounter.flags[1] = 0xFF;
				memImage->cfgCounter.flags[2] = 0xFF;
				memImage->cfgCounter.flags[3] = 0xFF;
		}
		
		// define FEE transaction type
		ADI_FEE_TRANSACTION flashTran = {
				.pWriteAddr = (uint32_t*)HART_FLASH_ADDR,
				.pWriteData = (uint32_t*)pageImage,
			  .nSize      = WRITE_LEN,  
			  .bUseDma    = false   		
		};
		
		// blocking write
		adi_fee_Write(hFlash, &flashTran , &err);
}


