#ifndef HART_LIB_H
#define HART_LIB_H

#include "hart_settings.h"
#include "../common.h"
#include "../DustNetworks/dn_endianness.h"

//=========================== define ==========================================

#define DEMO_DAMPING                   10 // damping value in seconds
#define HART_EXPANDED_DEVICE_TYPE      0xE28E
#define HART_DEVICE_ID                 0x000006
#define HART_MANUFACTURER              0x0004 // Analog Devices

#define HART_PROTOCOL_MAJOR_REV        7
#define HART_DEVICE_REV                1
#define HART_SOFTWARE_REV              1
#define HART_HARDWARE_REV              1

// supported number of bursts and events

#define HART_BURST0 0
#define HART_BURST1 1
#define HART_BURST2 2
#define HART_BURST3 3
#define HART_BURST_ALL             HART_BURST_NUM




#define HART_LONG_TAG                  "Tomas Rezucha, WH-mote" // 32 bytes max 
#define HART_TAG                       "ADI0"                   // 6  bytes max


#define HART_MAX_FRAME_LENGTH           128u // TODO change to actual value

// flage returned by hart_lib_cmdTerminatedByMCU when answer must not be generated
#define HART_LIB_DO_NOT_ANSWER            (-1)
#define HART_LIB_BUF_OVF                  (-2)
#define HART_LIB_FSM_NOT_READY            (-3)

//==== response codes HCF_SPEC-307
#define HART_RC_SUCESS                 0 
// 1 NOT DEFINED                       
#define HART_RC_INVALID_SELECTION      2
#define HART_RC_PARAM_TOO_LARGE        3
#define HART_RC_PARAM_TOO_SMALL        4
#define HART_RC_TOO_FEW_DATA           5
#define HART_RC_DEVICE_SPECIFIC_ERR    6  // must return 'device malfunction' and 'more status availible' in status byte
#define HART_RC_WRITE_PROTECTED        7
#define HART_RC_MULTI_WARNING_8        8
#define HART_RC_MULTI_ERR_9            9
#define HART_RC_MULTI_ERR_10          10
#define HART_RC_MULTI_ERR_11          11
#define HART_RC_MULTI_ERR_12          12
#define HART_RC_MULTI_ERR_13          13
#define HART_RC_MULTI_WARNING_14      14
#define HART_RC_MULTI_ERR_15          15
#define HART_RC_ACCESS_RESTRICTED     16
#define HART_RC_INVALID_DEV_VAR       17
#define HART_RC_INVALID_UNITS         18
#define HART_RC_DEV_VAR_NOT_ALLOWED   19
#define HART_RC_INVALID_EXTD_CMD_NUM  20
#define HART_RC_INVALIS_IO_CARD_NUM   21
#define HART_RC_INVALID_CHAN_NUM      22
#define HART_RC_SUBDEV_RESP_ERR       23
// 24-27 NOT DEFINED                  
#define HART_RC_MULTI_ERR_28          28 
#define HART_RC_MULTI_ERR_29          29
#define HART_RC_MULTI_WARNING_30      30
#define HART_RC_MULTI_WARNING_31      31
#define HART_RC_BUSY                  32
#define HART_RC_DELAYED_RESP_INIT     33
#define HART_RC_DELAYED_RESP_RUNNING  34
#define HART_RC_DELAYED_RESP_DEAD     35
#define HART_RC_DELAYED_RESP_CONFLICT 36
// 37-59 NOT DEFINED
#define HART_RC_PAYLOAD_TOO_LONG      60
#define HART_RC_NO_BUFF_AVAILIBLE_61  61
#define HART_RC_NO_BUFF_AVAILIBLE_62  62
#define HART_RC_PRIORITY_TOO_LOW      63
#define HART_RC_CMD_NOT_IMPLEMENTED   64
#define HART_RC_MULTI_ERR_65          65
#define HART_RC_MULTI_ERR_66          66
#define HART_RC_MULTI_ERR_67          67
#define HART_RC_MULTI_ERR_68          68
#define HART_RC_MULTI_ERR_69          69
#define HART_RC_MULTI_ERR_70          70
#define HART_RC_MULTI_ERR_71          71
#define HART_RC_MULTI_ERR_72          72
// END, THE REST IS NOT DEFINED IN HCF_SPEC307

//=========================== typedef =========================================

typedef enum
{
    HART_MASTER_SECONDARY       = 0,
    HART_MASTER_PRIMARY         = 1,
		HART_MASTER_GATEWAY         = 2,
		HART_MASTER_NETWORK_MANAGER = 3
} hart_lib_master_t;

typedef struct
{
		uint8_t  cmdId[2];
		uint8_t  dataLen;
		uint8_t  data[HART_MAX_FRAME_LENGTH];
} hart_lib_wirelessPacket_t;

typedef struct
{
		uint8_t  burstNum;
		uint32_t dataPeriod;
		uint8_t  dataLen;
		uint8_t  data[HART_MAX_FRAME_LENGTH];
} hart_lib_wirelessBurst_t;

//=== callback signature
typedef void (*hart_lib_wirelessBurst_cbt) (void);

//=========================== prototypes ======================================

int16_t hart_lib_cmdTerminatedByMCU(uint16_t cmdId, uint8_t *in, uint8_t inLen, uint8_t *out, uint8_t outLen, hart_lib_master_t master);
int16_t hart_lib_cmdTerminatedByMote(uint16_t cmdId, uint8_t *in, uint8_t inLen, uint8_t *out, uint8_t outLen);
uint8_t hart_lib_getStatus(hart_lib_master_t master);
uint8_t hart_lib_getExtendedStatus(void);
void    hart_lib_incCfgChangeCnt(void);
int8_t  hart_lib_enableBurst(hart_lib_wirelessBurst_cbt Cb , hart_lib_wirelessBurst_t* buffer, uint8_t burstNum, bool enable);

void    hart_lib_getUniqueAddress(uint8_t *uniqueAddr);
uint8_t hart_lib_getPollingAddress(void);
void    hart_lib_getMACaddress(uint8_t *MACaddr);


#endif
