#ifndef APP_H
#define APP_H

#include "common.h"

#include "hart/hart_settings.h"

//=========================== defines =========================================

//==== APP return codes
#define APP_OK                    0
#define APP_NOUPDATE              1
#define APP_ERROR                 2

//==== Device variables indexing
#define HART_PRIMARY_VAR          0
#define HART_SECONDARY_VAR        1
#define HART_TERTIARY_VAR         2
#define HART_QUATERNARY_VAR       3

//==== common tables HCF_SPEC-183 (the rest is in hart_lib.h)

// Table 2: Unit codes
#define HART2_UNIT_kPa            12
#define HART2_UNIT_Celsius        32
#define HART2_UNIT_mA             39
#define HART2_UNIT_PerCent        57
#define HART2_UNIT_Volt           58
#define HART2_UNIT_g              170     // gravital acceleration
#define HART2_UNIT_NONE           251

// Table 21: Device variable classification codes
#define HART21_CLASS_NONE         0
#define HART21_CLASS_TEMPERATURE  64
#define HART21_CLASS_PRESSURE     65
#define HART21_CLASS_EMF          83 // electromotive force
#define HART21_CLASS_ACCELERATION 96 
#define HART21_CLASS_MISC         111 

//==== Device variables definitions

// Primary variable
#define HART_PRIMARY_UNIT         HART2_UNIT_g
#define HART_PRIMARY_CLASS        HART21_CLASS_ACCELERATION

#define DEMO_FULL_SCALE (2)
#define DEMO_ZERO_SCALE    (-2)

// Secondary variable             
#define HART_SECONDARY_UNIT       HART2_UNIT_g
#define HART_SECONDARY_CLASS      HART21_CLASS_ACCELERATION
                                  
// Tertiary variable              
#define HART_TERTIARY_UNIT        HART2_UNIT_g
#define HART_TERTIARY_CLASS       HART21_CLASS_ACCELERATION
                                  
// Quaternary variable            
#define HART_QUATERNARY_UNIT      HART2_UNIT_Celsius
#define HART_QUATERNARY_CLASS     HART21_CLASS_TEMPERATURE  

//=========================== typedef =========================================

typedef struct
{
		float   value;
		float   valuePercent;
		uint8_t unit;           // HCF_SPEC-183, Table 2
		uint8_t classification;	// HCF_SPEC-183, Table 21
		bool    newData;        // flag
		bool    isOffLimits; 
} app_deviceVar_t;

//=========================== prototype =======================================

//==== app
int16_t               app_enableMeasurement(uint8_t varNum, bool enable);
app_deviceVar_t*      app_getDeviceVar(uint8_t varNum);
float                 app_getLoopCurrent(void);

#endif

