#include "fsm.h"
#include "gpio.h"

#include <drivers/pwr/adi_pwr.h>
#include <drivers/xint/adi_xint.h>

//=========================== prototype =======================================

int32_t gpio_initPinMux(void);

//=========================== variables =======================================

uint8_t  xint_DevMem[ADI_XINT_MEMORY_SIZE];  // Memory required for XINT driver
uint8_t  gpio_DevMem[ADI_GPIO_MEMORY_SIZE];  // Memory required for GPIO driver

//============================= main ==========================================

int main(void)
{
		uint32_t clock;
	
//======================== initialisation =====================================

    // Initialize power service
    if (adi_pwr_Init() != ADI_PWR_SUCCESS)
		{		
			  DEBUG_MSG("Power service init FAIL!\nExiting...\n");
        return 1;
		}
		
		// Initialize external interrupt service
		if (adi_xint_Init(xint_DevMem, ADI_XINT_MEMORY_SIZE) != ADI_XINT_SUCCESS)
		{		
			  DEBUG_MSG("XINT service init FAIL!\nExiting...\n");
        return 1;
		}
	
		// pin multiplexer init
    gpio_initPinMux();
		
		// send init MSG
		DEBUG_MSG(MSG_IDN);
		
		// indicated HCLK clock, just to be sure
		adi_pwr_GetClockFrequency(ADI_CLOCK_HCLK , &clock);
		DEBUG_MSG("HCLK = %d Hz\n", clock); 
		
		// run fsm
		fsm_enterState(0, FSM_STATE_START); // delay has no effect here, as the initialization is not scheduled	
}


void _sys_exit(int return_code)
{
label:
    goto label;
}

//============================= private =======================================

/*
 * Initialize the Port Control MUX Registers
 */
int32_t gpio_initPinMux(void)
{
    // init adi gpio driver
    adi_gpio_Init(gpio_DevMem, ADI_GPIO_MEMORY_SIZE);

    /* PORTx_MUX registers */
    *pREG_GPIO0_CFG = I2C0_SCL_PORTP0_MUX | I2C0_SDA_PORTP0_MUX
                      | UART0_TX_PORTP0_MUX | UART0_RX_PORTP0_MUX// | UART0_SOUT_EN_PORTP0_MUX
                      | SWD0_SWD0_CLK_PORTP0_MUX | SWD0_SWD0_DATA_PORTP0_MUX;
    *pREG_GPIO1_CFG = SPI1_CLK_PORTP1_MUX | SPI1_MOSI_PORTP1_MUX
                      | SPI1_MISO_PORTP1_MUX | SPI2_CLK_PORTP1_MUX | SPI2_MOSI_PORTP1_MUX
                      | SPI2_MISO_PORTP1_MUX | UART1_TX_PORTP1_MUX | SYS_BMODE_SYS_BMODE0_PORTP1_MUX;
    *pREG_GPIO2_CFG = SPI2_CS_2_PORTP2_MUX | RTC_RTC_SS1_PORTP2_MUX //| SYS_WAKEUP_SYS_WAKE3_PORTP2_MUX 
                      | UART1_RX_PORTP2_PIN0_MUX;// | UART1_RX_PORTP2_PIN13_MUX;
	
		// default pin states
		gpio_ledOff();

    // enable desired outputs
    adi_gpio_OutputEnable(ADI_GPIO_PORT0, ADI_GPIO_PIN_0, true);
	  adi_gpio_OutputEnable(ADI_GPIO_PORT1, ADI_GPIO_PIN_13, true);
	
		// and inputs
		adi_gpio_InputEnable(ADI_GPIO_PORT2, ADI_GPIO_PIN_1, true);
	

    return 0;
}

